import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

/**
 * Diese Klasse repraesentiert das Hauptfenster der Praesenzaufgabe
 * zum Umwandeln von Strings in Grossbuchstaben.
 *
 * @author Annabelle Klarl
 */
public class UpperCaseFrame extends JFrame implements ActionListener {
  private JButton anzahlstringsButton;

  private JTextArea ausgabeBereich;

  public static final String[] TESTARRAY = {null, "", "test"};

  /** In diesem Programmstueck wird das Fenster erzeugt */
  public UpperCaseFrame() {
    /* In der Kopfleiste des Fenster steht "UpperCaseFrame" */
    this.setTitle("UpperCaseFrame");

    /* Das Fenster ist 500 Pixel breit und 200 Pixel hoch. */
    this.setSize(500, 200);

    /* Hier werden alle Buttons erzeugt. */
    this.anzahlstringsButton =
        new JButton("Strings in Grossbuchstaben umwandeln");

    /* Hier wird der Ausgabe-Bereich erzeugt. */
    this.ausgabeBereich = new JTextArea(60, 100);
    JScrollPane scrollPane = new JScrollPane(this.ausgabeBereich);
    scrollPane.setHorizontalScrollBarPolicy(
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

    /*
     * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
     * Interaktionselemente (wie eine TextArea oder ein Button) platziert
     * werden koennen.
     */
    Container contentPane = this.getContentPane();
    contentPane.setLayout(new GridLayout(2, 1));
    /* Hier wird der Button platziert. */
    contentPane.add(this.anzahlstringsButton);
    /* Hier wird der Ausgabebereich platziert. */
    contentPane.add(scrollPane);

    /*
     * Hier wird der Frame als Listener fuer Knopfdruck-Ereignisse bei jedem
     * der Buttons registriert.
     */
    this.anzahlstringsButton.addActionListener(this);

    /*
     * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
     * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
     */
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }

  @Override
  public void actionPerformed(ActionEvent e) {
    // je nach Button entsprechende Methode ausfn
    Object source = e.getSource();
    if (source == this.anzahlstringsButton) {
      this.umwandelnStringsE();
    }
  }

  /**
   * implementiert die Funktionalitaet
   * des "Strings in Grossbuchstaben umwandeln"-Buttons
   */
  private void umwandelnStrings() {
    String einlesenAnzahlStrings =
        JOptionPane.showInputDialog("Parameter \"num\" eingeben: ");
    int num = Integer.parseInt(einlesenAnzahlStrings);

    this.ausgabeBereich.setText("");
    this.printUpperCaseForArray(TESTARRAY, num);
  }

  /**
   * implementiert die Funktionalitaet
   * des "Strings in Grossbuchstaben umwandeln"-Buttons
   */
  private void umwandelnStringsE() {
    String einlesenAnzahlStrings =
        JOptionPane.showInputDialog("Parameter \"num\" eingeben: ");
    int num = Integer.parseInt(einlesenAnzahlStrings);

    if ((num >= 0) && (num <= TESTARRAY.length)) {
      this.ausgabeBereich.setText("");
      this.printUpperCaseForArray(TESTARRAY, num);
    } else {
      this.ausgabeBereich.setText("Ungueltige Anzahl " + num);
    }
  }

  /**
   * wandelt einen String in Grossbuchstaben um
   *
   * @param s
   * @return
   */
  public String printUpperCase(String s)
        throws NullStringException, EmptyStringException {
    if (s == null) {
      throw new NullStringException();
    }
    if (s.length() == 0) {
      throw new EmptyStringException();
    }

    return s.toUpperCase();
  }

  public void printUpperCaseForArray(String[] array, int num) {
    printUpperCaseForArrayD(array, num);
  }

  /**
   * wandelt ein Array von Strings in Grossbuchstaben um
   *
   * @param array
   * @param num
   * @return
   */
  public void printUpperCaseForArrayB(String[] array, int num) {
    for (int i = 0; i < num; i++) {
      try {
        this.ausgabeBereich.append(this.printUpperCase(array[i]) + "\n");
      } catch (Exception e) {
        this.ausgabeBereich.append("Exception bei Versuch " + (i + 1) + "\n");
      }
    }
    this.ausgabeBereich.append("Fertig!\n");
  }

  /**
   * wandelt ein Array von Strings in Grossbuchstaben um
   *
   * @param array
   * @param num
   * @return
   */
  public void printUpperCaseForArrayC(String[] array, int num) {
    for (int i = 0; i < num; i++) {
      try {
        this.ausgabeBereich.append(this.printUpperCase(array[i]) + "\n");
      } catch (NullStringException e) {
        this.ausgabeBereich.append("text[" + i + "] war null!\n");
      } catch (EmptyStringException e) {
        this.ausgabeBereich.append("text[" + i + "] war leer!\n");
      }
    }
    this.ausgabeBereich.append("Fertig!\n");
  }

  /**
   * wandelt ein Array von Strings in Grossbuchstaben um
   *
   * @param array
   * @param num
   * @return
   */
  public void printUpperCaseForArrayD(String[] array, int num) {
    for (int i = 0; i < num; i++) {
      try {
        this.ausgabeBereich.append(this.printUpperCase(array[i]) + "\n");
      } catch (NullStringException e) {
        this.ausgabeBereich.append("text[" + i + "] war null!\n");
      } catch (EmptyStringException e) {
        this.ausgabeBereich.append("text[" + i + "] war leer!\n");
      } catch (ArrayIndexOutOfBoundsException e) {
        this.ausgabeBereich.append("Fehler! Falscher Arrayzugriff!\n");
      } finally {
        this.ausgabeBereich.append("Nach Versuch " + (i + 1) + "\n");
      }
    }
    this.ausgabeBereich.append("Fertig!\n");
  }
}
